package ru.scancode.GodexPrintTest;

import android.os.Bundle;
import android.util.SparseArray;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.google.android.material.snackbar.Snackbar;

public class MainActivity extends AppCompatActivity {
    private int nActivePage=0;
    private page_template tmpl;
    private EditText editQuantity;
    private Button btnPrint;
    private Button btnCalibrate;
    private Menu m_menu;
    private printAdapter printService;
    private FragmentActivity m_context;
    private static AbstractPage last_page;
    private MenuItem menu_item;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_client);

        editQuantity=findViewById(R.id.editCount);
        btnPrint=findViewById(R.id.btnPrint);
        btnCalibrate=findViewById(R.id.btnCalibrate);

        tmpl=new page_template(getSupportFragmentManager(), this);

        tmpl.addPage(new pagePrintTemplate(),           getString(R.string.print_template));
        tmpl.addPage(new pagePrintCmdFile(),            getString(R.string.print_cmd_file));
        tmpl.addPage(new pagePrintCustomCommandPage(),  getString(R.string.print_custom_cmd));
        if(BuildConfig.DEBUG){
            tmpl.addPage(new debugPage(), "debug");
        }

        ViewPager viewPager =findViewById(R.id.pager);
        viewPager.setAdapter(tmpl);
        viewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageSelected(int position) {
                nActivePage= position;
                boolean bEnable=false;
                switch(nActivePage){
                    case 0:
                    bEnable=true;
                        break;
                    case 2:
                        btnPrint.setText(R.string.execute);
                        break;
                    default:
                        btnPrint.setText(R.string.print);
                        break;
                }

                if(menu_item !=null) {
                    menu_item.setEnabled(bEnable);
                }
            }

            @Override
            public void onPageScrollStateChanged(int state) {

            }

            @Override
            public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {
                nActivePage=position;
            }
        });

        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN);


        init();
        if(BuildConfig.DEBUG){
            viewPager.setCurrentItem(tmpl.getCount()-1);
        }
    }

    private void init(){
        printService= printAdapter.getInstance(this);
        printService.setReceiver();
    }

    private void close(){
        printService= printAdapter.getInstance(this);
        printService.removeReceiver();
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.act_menu, menu);
        m_menu=menu;
        menu_item = m_menu.findItem(R.id.menuRefreshTemplates);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch(item.getItemId()){
            case R.id.menuRefreshTemplates:
                printService.getSavedTemplates(pagePrintTemplate.adapter);
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onPause() {
        super.onPause();
        close();
    }

    @Override
    protected void onResume() {
        super.onResume();
        if(getSupportFragmentManager().isStateSaved()){
            getSupportFragmentManager().popBackStackImmediate();
        }
        init();
    }

    public void onBtnCalibrateClick(View view){
        printService.calibrateSensor();
    }

    public void onBtnGetTemplatesClick(View view){
        printService.getSavedTemplates(pagePrintTemplate.adapter);
    }

    public static AbstractPage getLastPage(){
        return last_page;
    }

    public void onBtnPrintClick(View view){
        last_page=tmpl.getPage(nActivePage);
        if(last_page==null) {
            utils.logger.write("not page found");
            return;
        }

        int nQuantity=1;
        try{
            nQuantity=Integer.parseInt(editQuantity.getText().toString());
        }
        catch (NumberFormatException e){
            nQuantity=1;
        }

        last_page.setQuantity(nQuantity);

        SparseArray<AbstractPage.variable> vars=last_page.getVariables();
        if(vars.size()>0){
            last_page.PromptVariables(vars);
        }
        else {
            String data = last_page.getPrintData();

            if (data == null || data.isEmpty() || nQuantity <= 0) {
                String msg = getString(R.string.no_data);
                utils.logger.write(msg);
                Snackbar.make(editQuantity, msg, Snackbar.LENGTH_LONG);
            }
            else{
                printService.sendServiceCmd(data);
            }
        }
    }

    private class page_template extends FragmentPagerAdapter {
        private SparseArray<AbstractPage> pages= new SparseArray<>();

        page_template(FragmentManager fm, FragmentActivity context) {
            super(fm, BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT);
            m_context=context;
        }

        void addPage(AbstractPage page, String title) throws IllegalStateException {
            if(pages.indexOfValue(page)<0) {
                Bundle args= new Bundle();
                args.putString(AbstractPage.ARG_TITLE, title);

                page.setArguments(args);
                pages.append(pages.size(), page);
            }
            else{
                throw new IllegalStateException(getString(R.string.key_already_exists));
            }
        }

        @Nullable
        public AbstractPage getPage(int position){
            AbstractPage result=null;
            if(pages.indexOfKey(position)>=0){
                result=pages.get(position);
            }
            return result;
        }

        @Override
        public int getCount() {
            return pages.size();
        }

        @Override
        public Fragment getItem(int position) {
            AbstractPage result=null;

            if(pages.indexOfKey(position)>=0){
                result=pages.get(position);
            }
            return result;
        }

        @Override
        public CharSequence getPageTitle(int position) {
            AbstractPage page=pages.get(position);
            String title="";
            if(page!=null){
                title=page.getTitle();
            }
            return title;
        }
    }

}
