package ru.scancode.GodexPrintTest;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.util.SparseArray;
import android.widget.ArrayAdapter;
import android.widget.Toast;

import androidx.fragment.app.FragmentActivity;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Timer;
import java.util.TimerTask;

import static ru.scancode.GodexPrintTest.data_packer.EXTRA_ANSWER;
import static ru.scancode.GodexPrintTest.data_packer.EXTRA_DATA;
import static ru.scancode.GodexPrintTest.data_packer.EXTRA_RESPONSE;
import static ru.scancode.GodexPrintTest.data_packer.PRINT_ACTION;
import static ru.scancode.GodexPrintTest.data_packer.resp.RESP_FAIL;
import static ru.scancode.GodexPrintTest.data_packer.resp.RESP_OK;

//Класс для взаимодействия со службой печати
//====Singleton=====
public class printAdapter {
    private wait_dlg d;
    private AlertDialog dlg;
    private static Timer timer;
    private final static int TIME_TO_WAIT=10;
    private static int nTimeToWait=0;
    private static ArrayAdapter<pagePrintTemplate.template_label> m_adapter;
    private FragmentActivity context;
    private boolean bTimeOut=false;

    private static printAdapter pThis;
    private Intent serv= new Intent();
    private String SERVICE_PACKAGE="ru.scancode.GodexPrintService";

    public static printAdapter getInstance(FragmentActivity context){
        if(pThis==null){
            pThis= new printAdapter(context);
        }
        return pThis;
    }

    private printAdapter(FragmentActivity context) {
        this.context = context;
    }

    public void setReceiver(){
        if(context!=null) {
            //LocalBroadcastManager.getInstance(context).registerReceiver(m_receiver, new IntentFilter(PRINT_ACTION));
            context.registerReceiver(m_receiver, new IntentFilter(PRINT_ACTION));
        }
    }

    public void removeReceiver(){
        if(context!=null){
            //LocalBroadcastManager.getInstance(context).unregisterReceiver(m_receiver);
            context.unregisterReceiver(m_receiver);
        }
    }

    public void getSavedTemplates(ArrayAdapter<pagePrintTemplate.template_label> adapter){
        try {
            m_adapter=adapter;

            data_packer packet= new data_packer();
            packet.setCommand(data_packer.commands.CMD_GET_SAVED_TEMPLATES);

            serv.putExtra(data_packer.EXTRA_DATA, packet.toString());

            start(serv);

        } catch (Exception e) {
            utils.logger.write(e.getMessage());

        }
    }

    public void calibrateSensor(){
        data_packer dPacket=new data_packer();
        dPacket.setCommand(data_packer.commands.CMD_SEND_CMD);
        dPacket.addParam(data_packer.parameters.PARAM_CMD, data_packer.values.VAL_DATA, null, "~S,SENSOR");

        serv.putExtra(EXTRA_DATA, dPacket.toString());
        start(serv);
    }

    public void sendServiceCmd(String data) {
        serv.putExtra(data_packer.EXTRA_DATA, data);
        start(serv);
    }

    public void showWaitDialog(Context context){
        dlg = new ProgressDialog.Builder(context)
        .setTitle(R.string.app_name)
        .setMessage(R.string.please_wait)
        .setCancelable(false)
        .create();

        if(!((Activity)context).isFinishing()) {
            dlg.show();
        }
    }

    public void dismissWaitDialog(){
        if(dlg != null){
            dlg.dismiss();
            dlg = null;
        }
    }

//    public void startWaitDlg() {
//        FragmentManager fm = context.getSupportFragmentManager();
//        if(fm!=null) {
//            d=(wait_dlg) fm.findFragmentById(R.layout.wait_layout);
//            if (d == null) {
//                d = new wait_dlg();
//            }
//            try {
//                FragmentTransaction ft = fm.beginTransaction();
//                ft.add(d, "");
//                ft.commit();
//
//                //d.show(context.getSupportFragmentManager(), "");
//            } catch (Exception e) {
//                utils.logger.write(e.toString() + " " + e.getMessage());
//            }
//        }
//    }

//    public void stopWaitDlg(){
//        try {
//            if (d != null) {
//                FragmentManager fm = context.getSupportFragmentManager();
//                if(fm!=null) {
//                    FragmentTransaction ft=fm.beginTransaction();
//                    ft.remove(d);
//                    ft.commitAllowingStateLoss();
//                    //d.dismissAllowingStateLoss();
//                }
//            }
//        }
//        catch (Exception e){
//            e.printStackTrace();
//        }
//    }

    public void setTimeout(int nTimeOut){
        if((nTimeOut/100)>1) {
            nTimeToWait = nTimeOut;
        }
        else{
            nTimeToWait=nTimeOut*1000;
        }
    }

    //Сюда приходят ответы от GodexPrintService
    private BroadcastReceiver m_receiver= new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if(intent==null)
                return;

            if(!Objects.equals(intent.getAction(), PRINT_ACTION))
                return;

            if(timer!=null){
                timer.cancel();
                timer.purge();
            }
            dismissWaitDialog();
            //stopWaitDlg();

            //Признак ошибки:
            //0 - ОК
            //1 - ОШИБКА
            int resp=1;
            try {
                resp = intent.getIntExtra(EXTRA_RESPONSE, RESP_FAIL.ordinal());
            }
            catch (Exception e){
                String err=context.getString(R.string.unknown_service_answer);
                err+=e.getMessage();

                utils.logger.write(err);
                Toast.makeText(context, err, Toast.LENGTH_SHORT).show();
                return;
            }

            List<data_packer.InnerParam> params=new ArrayList<>();

            String answ_str =intent.getStringExtra(EXTRA_ANSWER); //Строка ответа от службы
            if(answ_str!=null) {
                try {
                    //Для удобства разбора XML ответа, ипользуется класс data_packer
                    data_packer dPacker = new data_packer(answ_str);
                    params= dPacker.getParams();

                    //Успешное выполнение
                    if (resp == RESP_OK.ordinal()) {
                        data_packer.InnerCommand cmd = dPacker.getCommand();
                        if (cmd != null) {
                            switch (cmd.getType()) {
                                case CMD_GET_SAVED_TEMPLATES:
                                    List<AbstractPage.template_label> labels= parse_templates(params);
                                    m_adapter.clear();
                                    for(AbstractPage.template_label label: labels){
                                        m_adapter.add(label);
                                    }
                                    m_adapter.notifyDataSetChanged();
                                    answ_str="";
                                    break;
                                default:
                                    data_packer.InnerParam param= dPacker.getResult();
                                    if(param!=null) {
                                        data_packer.InnerParam.value val = param.getValue(data_packer.values.VAL_DATA);
                                        if (val != null) {
                                            answ_str = val.getValue();
                                        }
                                    }
                                    else{
                                        throw new Exception(context.getString(R.string.unknown_parameter));
                                    }
                                    break;
                            }
                        }
                        else{
                            throw new Exception(context.getString(R.string.unknown_command));
                        }

                    //Ошибка
                    } else if (resp==RESP_FAIL.ordinal()) {
                        data_packer.InnerParam param = dPacker.getResult();
                        if (param != null) {
                            data_packer.InnerParam.value val = param.getValue(data_packer.values.VAL_ERROR);
                            if (val != null) {
                                answ_str = val.getValue();
                                //разбор кодов ошибок
                                switch(val.getErrCode()){
                                    case ERR_NO_VARS:
                                        AbstractPage page=MainActivity.getLastPage();
                                        if(page!=null){
                                            List<AbstractPage.template_label> labels=parse_templates(params);
                                            if(labels.isEmpty()){
                                                throw new Exception(answ_str);
                                            }
                                            page.PromptVariables(labels.get(0).getVariables());
                                        }
                                        break;
                                    default:
                                        throw new Exception(answ_str);
                                        //break;
                                }
                            }
                        }
                        else{
                            throw new Exception(context.getString(R.string.unknown_parameter));
                        }
                    }
                } catch(Exception e){
                    utils.logger.write(e.getMessage());
                    answ_str = "error: " + e.getMessage();
                }

                if(answ_str!=null && !answ_str.isEmpty())
                    Toast.makeText(context, answ_str, Toast.LENGTH_SHORT).show();
            }
        }
    };

    private List<AbstractPage.template_label> parse_templates(List<data_packer.InnerParam> params) {
        List<AbstractPage.template_label> result= new ArrayList<>();
        for (data_packer.InnerParam param : params) {
            if (param.getType().equals(data_packer.parameters.PARAM_TEMPLATE)) {
                pagePrintTemplate.template_label label = new pagePrintTemplate.template_label(param.getName());
                SparseArray<data_packer.InnerParam.value> values = param.getValues();
                if (values != null) {
                    for (int i = 0; i < values.size(); i++) {
                        data_packer.InnerParam.value val = values.get(i);
                        if (val.getType().equals(data_packer.values.VAL_VARIABLE)) {
                            label.addVariable(new AbstractPage.variable(val.getValue(), val.getName(), val.getLength()));
                        }
                    }
                }
                result.add(label);
            }
        }
        return result;
    }

    private void start(final Intent serv){
        bTimeOut=false;
        serv.setAction(data_packer.PRINT_ACTION);
        serv.addCategory(Intent.CATEGORY_DEFAULT);
        serv.setPackage(SERVICE_PACKAGE);
        context.startService(serv);
        //startWaitDlg();
        showWaitDialog(context);

        //Таймер ожидания ответа от службы, что б не зависнуть на долго)
        timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public boolean cancel() {
                utils.logger.write("timer canceled");
                return super.cancel();
            }

            @Override
            public void run() {
                //bTimeOut=true;
                data_packer dPack= new data_packer();
                dPack.setCommand(data_packer.commands.CMD_NONE);
                dPack.addParam(data_packer.parameters.PARAM_RESULT, data_packer.values.VAL_ERROR, null, "timeout");

                serv.setAction(PRINT_ACTION);
                serv.putExtra(EXTRA_RESPONSE, data_packer.resp.RESP_FAIL.ordinal());
                serv.putExtra(EXTRA_DATA, dPack.toString());
                context.sendBroadcast(serv);
                dismissWaitDialog();
            }
        }, (nTimeToWait>0 ? nTimeToWait : TIME_TO_WAIT*1000));
    }

}
